@@ -31,7 +31,8 @@ except:
    
 class scrape(object):
         
-    def newspaper_stories(words, urls=None, display=True):
+    def newspaper_stories(words, search_type='or', search_level=0, urls=None, 
+                      display=True):
         if urls == None:
            news_urls = {
                 'huffington': 'http://huffingtonpost.com', 
@@ -48,80 +49,86 @@ class scrape(object):
                 'fox':        'http://www.foxnews.com'}
         else:
             news_urls = urls
-            
-        df_articles = pd.DataFrame(columns=['agency', 'url', 'length', 'keywords', 
-                                        'title', 'summary', 'story'])
+        
+        if search_level==0:
+            print("\nSearch Level 0: Quick search of URLs for:", words)
+        else:
+            print("\nSearch Level 1: ",
+                  "Deep download and search of all qualified articles for:",
+                  words)
+        print("")   
+        df_articles = pd.DataFrame(columns=['agency', 'url', 'length', 
+                                            'keywords', 'title', 'summary', 
+                                            'story'])
         n_articles  = {}
-        today = str(date.today())
+        today       = str(date.today())
         for agency, url in news_urls.items():
-            paper = newspaper.build(url, memoize_articles=False, \
+            paper   = newspaper.build(url, memoize_articles=False, \
                                    fetch_images=False, request_timeout=20)
             if display:
-                print("\n", paper.size(), "Articles available from " +\
-                      agency.upper()+" on "+today+" :")
-            n_articles_selected = 0
+                print("{:>6d} Articles available from {:<s} on {:<10s}:".
+                      format(paper.size(), agency.upper(), today))
             article_collection = []
             for word in words:
                 word = word.lower()
                 for article in paper.articles:
+                    url_lower = article.url.lower()
                     # Exclude articles that are in a language other then en
                     # or contains mostly video or pictures
-                    if article.url.find('.video/')>=0    or \
-                       article.url.find('/video') >=0    or \
-                       article.url.find('/picture') >=0  or \
-                       article.url.find('.pictures/')>=0 or \
-                       article.url.find('/photo') >=0    or \
-                       article.url.find('.photos/')>=0   or \
-                       article.url.find('espanol')>=0    or \
-                       article.url.find('.mx/' )>=0 or \
-                       article.url.find('/mx.' )>=0 or \
-                       article.url.find('.fr/' )>=0 or \
-                       article.url.find('/fr.' )>=0 or \
-                       article.url.find('.de/' )>=0 or \
-                       article.url.find('/de.' )>=0 or \
-                       article.url.find('.it/' )>=0 or \
-                       article.url.find('/it.' )>=0 or \
-                       article.url.find('.gr/' )>=0 or \
-                       article.url.find('/gr.' )>=0 or \
-                       article.url.find('.se/' )>=0 or \
-                       article.url.find('/se.' )>=0 or \
-                       article.url.find('.es/' )>=0 or \
-                       article.url.find('/es.' )>=0 :
-                         continue
+                    # search_level 0 only searchs the url for keywords
+                    # search_level 0 only downloads articles with a 
+                    # keyword in their url.
+                    if search_level == 0 and url_lower.find(word) < 0:
+                        continue
+                    if url_lower.find('.video/')   >=0 or \
+                       url_lower.find('/video')    >=0 or \
+                       url_lower.find('/picture')  >=0 or \
+                       url_lower.find('.pictures/')>=0 or \
+                       url_lower.find('/photo')    >=0 or \
+                       url_lower.find('.photos/')  >=0 or \
+                       url_lower.find('espanol')   >=0 or \
+                       url_lower.find('.mx/' )     >=0 or \
+                       url_lower.find('/mx.' )     >=0 or \
+                       url_lower.find('.fr/' )     >=0 or \
+                       url_lower.find('/fr.' )     >=0 or \
+                       url_lower.find('.de/' )     >=0 or \
+                       url_lower.find('/de.' )     >=0 or \
+                       url_lower.find('.it/' )     >=0 or \
+                       url_lower.find('/it.' )     >=0 or \
+                       url_lower.find('.gr/' )     >=0 or \
+                       url_lower.find('/gr.' )     >=0 or \
+                       url_lower.find('.se/' )     >=0 or \
+                       url_lower.find('/se.' )     >=0 or \
+                       url_lower.find('.es/' )     >=0 or \
+                       url_lower.find('/es.' )     >=0 :
+                       continue
                     if agency=='usa-today':
-                        if article.url.find('tunein.com') <0:
-                               article_collection.append(article.url)
+                        if url_lower.find('tunein.com') <0:
+                            article_collection.append(article.url)
                         continue
                     if agency=='huffington':
-                        if article.url.find('.com') >=0:
-                               article_collection.append(article.url)
+                        if url_lower.find('.com') >=0:
+                            article_collection.append(article.url)
                         continue
                     if agency=='cbs-news':
-                        if  article.url.find('.com') >=0 :
-                                # secure-fly are duplicates of http
-                                if article.url.find('secure-fly')>=0:
+                        if  url_lower.find('.com') >=0 :
+                            # secure-fly are duplicates of http
+                            if article.url.find('secure-fly')>=0:
                                      continue
-                                article_collection.append(article.url)
+                            article_collection.append(article.url)
                         continue
                     article_collection.append(article.url)
             n_to_review = len(article_collection)
             if display:
-                print("Number of articles reviewed from",agency,":", n_to_review)
-            j = 0
+                print("{:>6d} Selected for download and review. Note that".
+                      format(n_to_review))
+                print("       download requests can fail, and multiple")
+                print("       copies of the same article are common.")
+                print("       These are filtered and saved as a single copy.")
+            
             for article_url in article_collection:
-                j += 1
                 article = Article(article_url)
                 article.download()
-                m     = article_url.find(".com")
-                m_org = article_url.find(".org")
-                if m_org>m:
-                    m = m_org
-                m += 5
-                k = len(article_url) - m
-                if k > 70:
-                    k=70
-                if display:
-                    print(j, " ", article_url[m:k+m])
                 n = 0
                 # Allow for a maximum of 5 download failures
                 stop_sec=1 # Initial max wait time in seconds
@@ -136,28 +143,38 @@ class scrape(object):
                         # Timeout for 5 seconds waiting for download
                         t0 = time()
                         tlapse = 0
-                        print("Waiting", stop_sec,"sec")
                         while tlapse<stop_sec:
                             tlapse = time()-t0
                         # Double wait time if needed for next exception
                         stop_sec = stop_sec+1
                 if n != 99:
-                    # download failed
+                    print("\nCannot download:", article_url[0:79])
+                    n_to_review -= 1
                     continue
                 article.nlp()
                 keywords = article.keywords
                 title    = article.title
                 summary  = article.summary
                 story    = article.text
-                # Verify the story contains at least one of the key words
                 found_it = False
                 story_lower_case = story.lower()
-                for i in range(len(words)):
-                    j = story_lower_case.find(words[i].lower())
-                    if j>= 0:
-                        found_it = True
-                        continue
-                if found_it:
+                if search_type == 'or':
+                    # Verify the story contains at least one of the key words
+                    for i in range(len(words)):
+                        j = story_lower_case.find(words[i].lower())
+                        if j>= 0:
+                            found_it = True
+                            continue
+                else: # search type 'and'
+                    found_it = True
+                    for i in range(len(words)):
+                        j = story_lower_case.find(words[i].lower())
+                        if j < 0:
+                            found_it = False
+                            continue
+                if not(found_it):
+                    n_to_review -= 1
+                else:
                     length   = len(story)
                     df_story = pd.DataFrame([[agency, url, length, keywords,
                                               title, summary, story]], 
@@ -170,31 +187,26 @@ class scrape(object):
                         # Verify this story is not already in df_articles
                         same_story = False
                         for i in range(df_articles.shape[0]):
-                            if title==df_articles['title'].iloc[i]:
+                            if story==df_articles['story'].iloc[i]:
                                 same_story   = True
                                 n_to_review -= 1
                                 continue
                         if not(same_story):
                             df_articles  = df_articles.append(df_story)
-                if display:
-                    print("Number of unique articles found in",agency,":", 
-                          n_to_review)
-            
-                n_articles_selected += 1
-            n_articles[agency] = [n_articles_selected, len(article_collection)]
-            if display:
-                ratio = str(n_articles_selected)+"/"+\
-                                    str(len(article_collection))
-                ratio = ratio + " articles selected from "+url.upper()
-                print(ratio)
+                print("=", end='')
+            n_articles[agency] = [n_to_review, len(article_collection)]
         if display:
             print("")
             for agency in news_urls:
-                ratio = str(n_articles[agency][0])+"/"+str(n_articles[agency][1])
+                ratio = str(n_articles[agency][0]) + "/" + \
+                        str(n_articles[agency][1])
                 ratio = ratio + " articles selected from "+agency.upper()
                 print(ratio)
-            print("\nTotal Articles Selected on "+today+":", df_articles.shape[0])
+            print("\nTotal Articles Collected on "+today+":", 
+                      df_articles.shape[0],'from', 
+                      df_articles['agency'].nunique(), "News Agencies.")
         return df_articles
+
     
     def clean_html(html):
         # First we remove inline JavaScript/CSS:
